@@ -49,6 +49,9 @@ struct _CalBackendWCAPPrivate {
 	gint timeout_id;
 	time_t timestamp;
 
+	GMutex *set_mode_lock;
+	GMutex *open_lock;
+	
 	gboolean is_dirty;
 	guint dirty_idle_id;
 	GMutex *idle_save_mutex;
@@ -1179,6 +1182,16 @@ cal_backend_wcap_finalize (GObject *object)
 			priv->instances = NULL;
 		}
 
+		if (priv->set_mode_lock) {
+			g_mutex_free (priv->set_mode_lock);
+			priv->set_mode_lock = NULL;
+		}
+
+		if (priv->open_lock) {
+			g_mutex_free (priv->open_lock);
+			priv->open_lock = NULL;
+		}
+
 		g_free (priv);
 		wcap->priv = NULL;
 	}
@@ -1194,6 +1207,7 @@ cal_backend_wcap_is_read_only (ECalBackendSync *backend, EDataCal *cal, gboolean
 	CalBackendWCAPPrivate *priv = wcap->priv;
 	gboolean hasWrite, hasDelete;
 	
+	g_return_val_if_fail (read_only != NULL, GNOME_Evolution_Calendar_InvalidObject);
 	g_return_val_if_fail (IS_CAL_BACKEND_WCAP (wcap), GNOME_Evolution_Calendar_OtherError);
 
 	d(printf("cal_backend_wcap_is_read_only(%p, %d)\n", backend, *read_only));
@@ -1222,6 +1236,7 @@ cal_backend_wcap_get_cal_address (ECalBackendSync *backend, EDataCal *cal, char
 	CalBackendWCAP *wcap = (CalBackendWCAP *) backend;
 	CalBackendWCAPPrivate *priv = wcap->priv;
 
+	g_return_val_if_fail (address != NULL, GNOME_Evolution_Calendar_InvalidObject);
 	g_return_val_if_fail (IS_CAL_BACKEND_WCAP (wcap), GNOME_Evolution_Calendar_OtherError);
 
 	*address = g_strdup (priv->account_email_address);
@@ -1236,6 +1251,7 @@ cal_backend_wcap_get_alarm_email_address (ECalBackendSync *backend, EDataCal *ca
 	CalBackendWCAPPrivate *priv = wcap->priv;
 	SunOneConnectionPreferences *prefs;
 	
+	g_return_val_if_fail (address != NULL, GNOME_Evolution_Calendar_InvalidObject);
 	g_return_val_if_fail (IS_CAL_BACKEND_WCAP (wcap), GNOME_Evolution_Calendar_OtherError);
 
 	d(printf("cal_backend_wcap_get_alarm_email_address(%p, %s)\n", backend, *address));
@@ -1258,6 +1274,7 @@ cal_backend_wcap_get_ldap_attribute (ECalBackendSync *backend, EDataCal *cal, ch
 {
 	CalBackendWCAP *wcap = (CalBackendWCAP *) backend;
 
+	g_return_val_if_fail (attribute != NULL, GNOME_Evolution_Calendar_InvalidObject);
 	g_return_val_if_fail (IS_CAL_BACKEND_WCAP (wcap), GNOME_Evolution_Calendar_OtherError);
 
 	*attribute = g_strdup ("icscalendar");
@@ -1270,6 +1287,8 @@ cal_backend_wcap_get_static_capabilities (ECalBackendSync *backend, EDataCal *ca
 	CalBackendWCAP *wcap = (CalBackendWCAP *) backend;
 	CalBackendWCAPPrivate *priv = wcap->priv;
 
+	g_return_val_if_fail (capabilities != NULL, GNOME_Evolution_Calendar_InvalidObject);
+
 	if (priv->so_cnc && IS_WCAP_2_0(wcap))
 		*capabilities = g_strdup (
 		 	CAL_STATIC_CAPABILITY_NO_TRANSPARENCY "," \
@@ -1426,6 +1445,7 @@ cal_backend_wcap_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_if
 	CalBackendWCAPPrivate *priv = wcap->priv;
 	char *type;
 	const char *uristr;
+	int ret;
 	
 	d(printf("cal_backend_wcap_open(%p, %d, %s, %s)\n", backend, only_if_exists, username, password));
 
@@ -1436,11 +1456,14 @@ cal_backend_wcap_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_if
 	uristr = e_cal_backend_get_uri (E_CAL_BACKEND (wcap));
 	d(g_message ("uri is %s", uristr));
 
+	g_mutex_lock (priv->open_lock);
+
 	priv->uri = g_strdup (uristr);
 	
 	priv->calid = sunone_util_get_calid_from_uri (uristr);
 	if (!priv->calid) {
 		cal_backend_wcap_clean_wcap (wcap, TRUE);
+		g_mutex_unlock (priv->open_lock);
 		return GNOME_Evolution_Calendar_NoSuchCal;
 	}
 
@@ -1462,17 +1485,21 @@ cal_backend_wcap_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_if
 		
 		if (!display_contents || !g_str_equal (display_contents, "1")) {
 			cal_backend_wcap_clean_wcap (wcap, TRUE);
+			g_mutex_unlock (priv->open_lock);
 			return GNOME_Evolution_Calendar_RepositoryOffline;
 		}
 
 		read_cache (wcap);
 
+		g_mutex_unlock (priv->open_lock);
 		return GNOME_Evolution_Calendar_Success;
 	}
 
 	read_cache (wcap);
+	ret = go_online (wcap);
+	g_mutex_unlock (priv->open_lock);
 
-	return go_online (wcap);
+	return ret;
 }
 
 static ECalBackendSyncStatus
@@ -1494,6 +1521,7 @@ cal_backend_wcap_remove (ECalBackendSync *backend, EDataCal *cal)
 	g_return_val_if_fail (priv->calid != NULL, GNOME_Evolution_Calendar_InvalidObject);
 	account = sunone_component_get_account_from_uri (global_sunone_component, priv->uri);
 	remove_source (account, priv->calid, FALSE);
+	sunone_folder_tree_sync ();
 	g_object_unref (G_OBJECT (account));
 	if (unlink (priv->cache_name)) {
 		g_object_unref (wcap);
@@ -1527,6 +1555,9 @@ cal_backend_wcap_send_objects (ECalBackendSync *backend, EDataCal *cal, const ch
 	icalproperty_method method;
 	CalBackendWCAP *wcap = CAL_BACKEND_WCAP (backend);
 
+	g_return_val_if_fail (users != NULL, GNOME_Evolution_Calendar_InvalidObject);
+	g_return_val_if_fail (modified_calobj != NULL, GNOME_Evolution_Calendar_InvalidObject);
+
 	d(printf("cal_backend_wcap_send_objects(%p, %s, %s)\n", backend, calobj, *modified_calobj));
 
 	if (!cal_backend_wcap_is_online (wcap))
@@ -1568,6 +1599,8 @@ cal_backend_wcap_get_default_object (ECalBackendSync *backend, EDataCal *cal, ch
 {
  	ECalComponent *comp;
  	
+	g_return_val_if_fail (object != NULL, GNOME_Evolution_Calendar_InvalidObject);
+
  	comp = e_cal_component_new ();
 
  	switch (e_cal_backend_get_kind (E_CAL_BACKEND (backend))) {
@@ -1596,8 +1629,9 @@ cal_backend_wcap_get_object (ECalBackendSync *backend, EDataCal *cal,
 	CalBackendWCAPPrivate *priv = wcap->priv;
 	ECalComponent *comp;
 
-	g_return_val_if_fail (IS_CAL_BACKEND_WCAP (wcap), GNOME_Evolution_Calendar_OtherError);
 	g_return_val_if_fail (uid != NULL, GNOME_Evolution_Calendar_InvalidObject);
+	g_return_val_if_fail (object != NULL, GNOME_Evolution_Calendar_InvalidObject);
+	g_return_val_if_fail (IS_CAL_BACKEND_WCAP (wcap), GNOME_Evolution_Calendar_OtherError);
 
 	d(printf("cal_backend_wcap_get_object(%p, %s, %s)\n", backend, uid, rid));
 
@@ -1629,8 +1663,9 @@ cal_backend_wcap_get_object_list (ECalBackendSync *backend, EDataCal *cal,
 	const char *uristr;
 	char *type;
 
-	g_return_val_if_fail (IS_CAL_BACKEND_WCAP (wcap), GNOME_Evolution_Calendar_OtherError);
 	g_return_val_if_fail (sexp != NULL, GNOME_Evolution_Calendar_InvalidObject);
+	g_return_val_if_fail (objects != NULL, GNOME_Evolution_Calendar_InvalidObject);
+	g_return_val_if_fail (IS_CAL_BACKEND_WCAP (wcap), GNOME_Evolution_Calendar_OtherError);
 
 	match_data.search_needed = TRUE;
 	match_data.query = sexp;
@@ -1824,6 +1859,9 @@ cal_backend_wcap_get_changes (ECalBackendSync *backend, EDataCal *cal, const cha
 	struct ChangeData data;
 	struct cbw_data cbwdata;
 
+	g_return_val_if_fail (adds != NULL, GNOME_Evolution_Calendar_InvalidObject);
+	g_return_val_if_fail (modifies != NULL, GNOME_Evolution_Calendar_InvalidObject);
+	g_return_val_if_fail (deletes != NULL, GNOME_Evolution_Calendar_InvalidObject);
 	d(printf("cal_backend_wcap_get_changes(%p, %s)\n", backend, change_id));
 
 	uristr = e_cal_backend_get_uri (E_CAL_BACKEND (backend));
@@ -1985,6 +2023,8 @@ cal_backend_wcap_set_mode (ECalBackend *backend, CalMode mode)
 		return;
 	}
 
+	g_mutex_lock (priv->set_mode_lock);
+
 	if (mode == CAL_MODE_REMOTE) {
 		if (priv->uri) {
 			account = sunone_component_get_account_from_uri (global_sunone_component, priv->uri);
@@ -1996,21 +2036,26 @@ cal_backend_wcap_set_mode (ECalBackend *backend, CalMode mode)
 		if (priv->so_cnc && sunone_connection_is_open (priv->so_cnc))
 			sunone_connection_logout (priv->so_cnc);
 		in_offline (wcap);
-		account = sunone_component_get_account_from_uri (global_sunone_component, priv->uri);
-		sunone_account_set_offline (account);
-		g_object_unref (account);
+		if (priv->uri) {
+			account = sunone_component_get_account_from_uri (global_sunone_component, priv->uri);
+			sunone_account_set_offline (account);
+			g_object_unref (account);
+		}
 	} else {
 		e_cal_backend_notify_mode (backend, GNOME_Evolution_Calendar_CalListener_MODE_NOT_SUPPORTED, cal_mode_to_corba (mode));
+		g_mutex_unlock (priv->set_mode_lock);
 		return;
 	}
 	
 	if (status != GNOME_Evolution_Calendar_Success) {
 		e_cal_backend_notify_mode (backend, GNOME_Evolution_Calendar_CalListener_MODE_NOT_SET, cal_mode_to_corba (mode));
+		g_mutex_unlock (priv->set_mode_lock);
 		return;
 	}
 	
 	priv->mode = mode;
 	e_cal_backend_notify_mode (backend, GNOME_Evolution_Calendar_CalListener_MODE_SET, cal_mode_to_corba (mode));
+	g_mutex_unlock (priv->set_mode_lock);
 }
 
 static icaltimezone *
@@ -2059,6 +2104,7 @@ cal_backend_wcap_get_timezone (ECalBackendSync *backend, EDataCal *cal,
 	icaltimezone *zone;
 	icalcomponent *vtzcomp;
 
+	g_return_val_if_fail (object != NULL, GNOME_Evolution_Calendar_InvalidObject);
 	g_return_val_if_fail (IS_CAL_BACKEND_WCAP (wcap), GNOME_Evolution_Calendar_OtherError);
 
 	zone = cal_backend_wcap_get_timezone_from_tzid (wcap, tzid, FALSE);
@@ -2142,5 +2188,8 @@ cal_backend_wcap_init (CalBackendWCAP *object)
 	priv->objects = g_hash_table_new (g_str_hash, g_str_equal);
 	priv->instances = g_hash_table_new (g_str_hash, g_str_equal);
 
+	priv->set_mode_lock = g_mutex_new ();
+	priv->open_lock = g_mutex_new ();
+
 	e_cal_backend_sync_set_lock (E_CAL_BACKEND_SYNC (wcap), TRUE);
 }
