@@ -1769,6 +1769,69 @@ sunone_connection_list_calids (SunOneConnection *connection)
 	return calids;
 }
 
+GList *
+sunone_connection_search_calids (SunOneConnection *connection, SunOneSearchField field, SunOneSearchBy by, const char *search_str)
+{
+	SunOneConnectionPrivate *priv = connection->priv;
+	SoupUri *get_uri;
+	SoupMessage *msg;
+	icalcomponent *icalcomp, *subcomp;
+	SunOneCalendarProperties *calprops;
+	GList *calids = NULL;
+	gint calid = 0, name = 0, primary_owner = 0;
+
+	g_return_val_if_fail (IS_SUNONE_CONNECTION (connection), NULL);
+
+	IS_CONNECTED (connection, NULL);
+
+	switch (field) {
+	case NAME_OR_ID:
+		calid = name = 1;
+		break;
+	case NAME:
+		name = 1;
+		break;
+	case PRIMARY_OWNER:
+		primary_owner = 1;
+		break;
+	case ID:	
+		calid = 1;
+		break;
+	}
+	/* prepare the URI */
+	NEW_URI (get_uri);
+	get_uri->path = g_strdup ("/search_calprops.wcap");
+	get_uri->query = g_strdup_printf ("id=%s&fmt-out=text/calendar&calid=%d&name=%d&primaryOwner=%d&searchOpts=%d&search-string=%s", priv->wcap_session_id, calid, name, primary_owner, by, search_str);
+
+	/* send the request to the server */
+	msg = sunone_message_new_from_uri (get_uri, SOUP_METHOD_GET);
+	sunone_message_send (connection, msg);
+
+	soup_uri_free (get_uri);
+
+	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
+		icalcomp = icalparser_parse_string (msg->response.body);
+		if (icalcomp) {
+			subcomp = icalcomponent_get_first_component (icalcomp, ICAL_VCALENDAR_COMPONENT);
+			while (subcomp) {
+				calprops = calprops_from_icalcomp (subcomp);
+				if (calprops)
+					calids = g_list_append (calids, calprops);
+
+				subcomp = icalcomponent_get_next_component (icalcomp, ICAL_VCALENDAR_COMPONENT);
+			}
+
+			icalcomponent_free (icalcomp);
+		} else
+			g_warning ("Error: could not parse response's body");
+	} else
+		g_warning ("Error: %s", msg->reason_phrase);
+
+	g_object_unref (msg);
+
+	return calids;
+}
+
 void
 sunone_connection_free_calprops (SunOneCalendarProperties *calprops)
 {
