@@ -25,15 +25,13 @@ extern char *evolution_dir;
 struct _SunOneSubscriptionDialogPrivate {
 	SunOneFolderTree *tree;
 	SunOneConnection *cnc;
-	GList *calids;
 	GList *matched_calids;
 	SunOneConnectionPreferences *prefs;
 	GHashTable *to_subscribe;
 	GladeXML *xml;
 	GtkWidget *main_table;
 	GtkTreeView *subscriptions_tree;
-	GtkWidget *subscribe_button;
-	GtkWidget *unsubscribe_button;
+	GtkWidget *search_field;
 	GtkWidget *search_by;
 	GtkWidget *search_string;
 	GtkWidget *search_button;
@@ -47,20 +45,6 @@ static void sunone_subscription_dialog_finalize (GObject *object);
 
 static GObjectClass *parent_class = NULL;
 
-enum {
-	ANY_FIELD_CONTAINS,
-	CALENDAR_ID_CONTAINS,
-	OWNER_CONTAINS,
-	DESCRIPTION_CONTAINS,
-};
-static const int searchby_map[] = {
-	ANY_FIELD_CONTAINS,
-	CALENDAR_ID_CONTAINS,
-	OWNER_CONTAINS,
-	DESCRIPTION_CONTAINS,
-	-1
-};
-
 static void
 sunone_subscription_dialog_class_init (SunOneSubscriptionDialogClass *klass)
 {
@@ -102,103 +86,63 @@ refresh_table (SunOneSubscriptionDialog *dialog)
 	}
 }
 
+static void
+free_matched_calids (SunOneSubscriptionDialogPrivate *priv)
+{
+	if (priv->matched_calids) {
+		g_list_foreach (priv->matched_calids, (GFunc) sunone_connection_free_calprops, NULL);
+		g_list_free (priv->matched_calids);
+		priv->matched_calids = NULL;
+	}
+}
+
 static void
 clear_search_cb (GtkButton *button, gpointer user_data)
 {
 	SunOneSubscriptionDialog *dialog = SUNONE_SUBSCRIPTION_DIALOG (user_data);
 	SunOneSubscriptionDialogPrivate *priv = dialog->priv;
-	GList *l;
 
 	g_return_if_fail (IS_SUNONE_SUBSCRIPTION_DIALOG (dialog));
 
-	gtk_combo_box_set_active ((GtkComboBox *) priv->search_by, ANY_FIELD_CONTAINS);
+	gtk_combo_box_set_active ((GtkComboBox *) priv->search_by, CONTAINS);
+	gtk_combo_box_set_active ((GtkComboBox *) priv->search_field, NAME_OR_ID);
 	gtk_entry_set_text (GTK_ENTRY (priv->search_string), "");
 
 	/* refresh the list */
-	g_list_free (priv->matched_calids);
-	priv->matched_calids = NULL;
-	for (l = priv->calids; l != NULL; l = l->next)
-		priv->matched_calids = g_list_append (priv->matched_calids, l->data);
-
-	gtk_widget_set_sensitive (priv->subscribe_button, FALSE);
-	gtk_widget_set_sensitive (priv->unsubscribe_button, FALSE);
-
+	free_matched_calids (priv);
 	refresh_table (dialog);
 }
 
-static gboolean
-find_string (const gchar *str, const gchar *substr)
-{
-	gchar *up_str, *up_substr;
-	gboolean found;
-
-	up_str = g_strdup (str);
-	g_ascii_strup (up_str, strlen (up_str));
-	up_substr = g_strdup (substr);
-	g_ascii_strup (up_substr, strlen (up_substr));
-
-	found = strstr (up_str, up_substr) ? TRUE : FALSE;
-
-	g_free (up_str);
-	g_free (up_substr);
-
-	return found;
-}
-
 static void
 do_search_cb (GtkButton *button, gpointer user_data)
 {
 	SunOneSubscriptionDialog *dialog = SUNONE_SUBSCRIPTION_DIALOG (user_data);
 	SunOneSubscriptionDialogPrivate *priv = dialog->priv;
 	GList *l;
-	gint by;
+	gint by, field;
 	const char *str;
 
 	g_return_if_fail (IS_SUNONE_SUBSCRIPTION_DIALOG (dialog));
 
-	g_list_free (priv->matched_calids);
-	priv->matched_calids = NULL;
+	/*free previous result*/
+	free_matched_calids (priv);
 
+	/*do new search*/
 	by = gtk_combo_box_get_active ((GtkComboBox *) priv->search_by);
+	field = gtk_combo_box_get_active ((GtkComboBox *) priv->search_field);
 	str = gtk_entry_get_text (GTK_ENTRY (priv->search_string));
+	if (str && *str != 0)
+		priv->matched_calids = sunone_connection_search_calids (priv->cnc, field, by, str);
 
-	for (l = priv->calids; l != NULL; l = l->next) {
-		SunOneCalendarProperties *calprops = l->data;
-		gint pos;
+	priv->prefs = sunone_connection_get_preferences (priv->cnc);
+	if (priv->prefs) {
+		for (l = priv->prefs->subscriptions; l != NULL; l = l->next) {
+			SunOneCalendarSubscription *sub = l->data;
 
-		pos = g_list_position (priv->matched_calids, g_list_find (priv->matched_calids, calprops));
-
-		switch (by) {
-		case ANY_FIELD_CONTAINS :
-			if (find_string (calprops->relative_calid, str) ||
-			    find_string (calprops->primary_owner, str) ||
-			    find_string (calprops->display_name ? calprops->display_name : "", str)) {
-				priv->matched_calids = g_list_append (priv->matched_calids, calprops);
-				continue;
-			}
-			break;
-		case CALENDAR_ID_CONTAINS :
-			if (find_string (calprops->relative_calid, str)) {
-				priv->matched_calids = g_list_append (priv->matched_calids, calprops);
-				continue;
-			}
-			break;
-		case OWNER_CONTAINS :
-			if (find_string (calprops->primary_owner, str)) {
-				priv->matched_calids = g_list_append (priv->matched_calids, calprops);
-				continue;
-			}
-			break;
-		case DESCRIPTION_CONTAINS :
-			if (find_string (calprops->display_name ? calprops->display_name : "", str)) {
-				priv->matched_calids = g_list_append (priv->matched_calids, calprops);
-				continue;
-			}
-			break;
+			g_hash_table_insert (priv->to_subscribe, g_strdup (sub->calid),
+					     g_strdup (sub->display_name ? sub->display_name : ""));
 		}
 	}
-	gtk_widget_set_sensitive (priv->subscribe_button, FALSE);
-        gtk_widget_set_sensitive (priv->unsubscribe_button, FALSE);
 
 	refresh_table (dialog);
 }
@@ -216,69 +160,24 @@ search_activated_cb (GtkComboBox *w, gpointer user_data)
 }
 
 static void
-subscribe_cb (GtkButton *button, gpointer user_data)
+subscribe_toggled (GtkCellRendererToggle *cell, gchar *path_str, gpointer data)
 {
-	SunOneSubscriptionDialog *dialog = SUNONE_SUBSCRIPTION_DIALOG (user_data);
+	SunOneSubscriptionDialog *dialog = SUNONE_SUBSCRIPTION_DIALOG (data);
 	SunOneSubscriptionDialogPrivate *priv = dialog->priv;
 	SunOneCalendarProperties *calprops;
-	GtkTreeSelection * selection;
 	GtkTreeModel *model;
-	GtkTreePath *path;
-	GtkTreeIter iter;
-	gint row;
-	GList *l;
-
-	g_return_if_fail (IS_SUNONE_SUBSCRIPTION_DIALOG (dialog));
-
-	selection = gtk_tree_view_get_selection (priv->subscriptions_tree);
-	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
-		return;
-	path = gtk_tree_model_get_path (model, &iter);
-	row = gtk_tree_path_get_indices (path)[0];
-	gtk_tree_path_free (path);
-
-	
-	g_return_if_fail (row >= 0 && row < g_list_length (priv->matched_calids));
-
-	l = g_list_nth (priv->matched_calids, row);
-	if (!l)
-		return;
-
-	calprops = l->data;
-	g_hash_table_insert (priv->to_subscribe, g_strdup (calprops->relative_calid),
-			     g_strdup (calprops->display_name ? calprops->display_name : ""));
-
-	gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-			    SUB_COLUMN,
-			    TRUE,
-			    -1);
-
-	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), GTK_RESPONSE_OK, TRUE);
-	gtk_widget_set_sensitive (priv->subscribe_button, FALSE);
-	gtk_widget_set_sensitive (priv->unsubscribe_button, TRUE);
-}
-
-static void
-unsubscribe_cb (GtkButton *button, gpointer user_data)
-{
-	SunOneSubscriptionDialog *dialog = SUNONE_SUBSCRIPTION_DIALOG (user_data);
-	SunOneSubscriptionDialogPrivate *priv = dialog->priv;
-	SunOneCalendarProperties *calprops;
-	GtkTreeSelection * selection;
-	GtkTreeModel *model;
-	GtkTreePath *path;
-	GtkTreeIter iter;
+	GtkTreeIter  iter;
+	GtkTreePath *path = gtk_tree_path_new_from_string (path_str);
+	gboolean subscribe;
 	gint row;
 	GList *l;
 	gpointer orig_key, orig_value;
 
-
 	g_return_if_fail (IS_SUNONE_SUBSCRIPTION_DIALOG (dialog));
 
-	selection = gtk_tree_view_get_selection (priv->subscriptions_tree);
-	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
-		return;
-	path = gtk_tree_model_get_path (model, &iter);
+	model = gtk_tree_view_get_model (priv->subscriptions_tree);
+	gtk_tree_model_get_iter (model, &iter, path);
+	gtk_tree_model_get (model, &iter, SUB_COLUMN, &subscribe, -1);
 	row = gtk_tree_path_get_indices (path)[0];
 	gtk_tree_path_free (path);
 
@@ -289,61 +188,23 @@ unsubscribe_cb (GtkButton *button, gpointer user_data)
 		return;
 
 	calprops = l->data;
-	if (g_hash_table_lookup_extended (priv->to_subscribe, calprops->relative_calid, &orig_key, &orig_value)) {
-		g_hash_table_remove (priv->to_subscribe, calprops->relative_calid);
-		g_free (orig_key);
-		g_free (orig_value);
-	}
-	gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-			    SUB_COLUMN,
-			    FALSE,
-			    -1);
-
-	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), GTK_RESPONSE_OK, TRUE);
-	gtk_widget_set_sensitive (priv->subscribe_button, TRUE);
-	gtk_widget_set_sensitive (priv->unsubscribe_button, FALSE);
-}
-
-static void
-subscription_selection_changed_cb (GtkTreeSelection *selection, gpointer user_data)
-{
-	SunOneSubscriptionDialog *dialog = SUNONE_SUBSCRIPTION_DIALOG (user_data);
-	SunOneSubscriptionDialogPrivate *priv = dialog->priv;
-	SunOneCalendarProperties *calprops;
-	GtkTreeModel *model;
-	GtkTreePath *path;
-	GtkTreeIter iter;
-	GList *l;
-	gint row;
-
-	g_return_if_fail (IS_SUNONE_SUBSCRIPTION_DIALOG (dialog));
-
-	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+	/* don't allow subscription from default calendar */
+	if (!strcmp (calprops->relative_calid, priv->prefs->default_calendar))
 		return;
-	path = gtk_tree_model_get_path (model, &iter);
-	row = gtk_tree_path_get_indices (path)[0];
-	gtk_tree_path_free (path);
-
-	g_return_if_fail (row >= 0 && row < g_list_length (priv->matched_calids));
-
-	l = g_list_nth (priv->matched_calids, row);
-	if (!l)
-		return;
-
-	calprops = (SunOneCalendarProperties *) l->data;
-	if (g_hash_table_lookup (priv->to_subscribe, calprops->relative_calid)) {
-		gtk_widget_set_sensitive (priv->subscribe_button, FALSE);
-		gtk_widget_set_sensitive (priv->unsubscribe_button, TRUE);
+	if (!subscribe) {
+		g_hash_table_insert (priv->to_subscribe, g_strdup (calprops->relative_calid),
+		g_strdup (calprops->display_name ? calprops->display_name : ""));
 	} else {
-		gtk_widget_set_sensitive (priv->subscribe_button, TRUE);
-		gtk_widget_set_sensitive (priv->unsubscribe_button, FALSE);
+		if (g_hash_table_lookup_extended (priv->to_subscribe, calprops->relative_calid, &orig_key, &orig_value)) {
+			g_hash_table_remove (priv->to_subscribe, calprops->relative_calid);
+			g_free (orig_key);
+			g_free (orig_value);
+		}
 	}
+	subscribe = !subscribe;
 
-	/* don't allow subscription/unsusbscription from default calendar */
-	if (!strcmp (calprops->relative_calid, priv->prefs->default_calendar)) {
-		gtk_widget_set_sensitive (priv->subscribe_button, FALSE);
-		gtk_widget_set_sensitive (priv->unsubscribe_button, FALSE);
-	}
+	gtk_list_store_set (GTK_LIST_STORE (model), &iter, SUB_COLUMN, subscribe, -1);
+	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), GTK_RESPONSE_OK, TRUE);
 }
 
 static void
@@ -352,13 +213,11 @@ sunone_subscription_dialog_init (SunOneSubscriptionDialog *object)
 	SunOneSubscriptionDialog *dialog = SUNONE_SUBSCRIPTION_DIALOG (object);
 	SunOneSubscriptionDialogPrivate *priv;
 	GtkWidget *label;
-	GtkTreeSelection *selection;
 
 	dialog->priv = g_new0 (SunOneSubscriptionDialogPrivate, 1);
 	priv = dialog->priv;
 
 	priv->cnc = NULL;
-	priv->calids = NULL;
 	priv->matched_calids = NULL;
 	priv->prefs = NULL;
 	priv->to_subscribe = g_hash_table_new (g_str_hash, g_str_equal);
@@ -377,12 +236,11 @@ sunone_subscription_dialog_init (SunOneSubscriptionDialog *object)
 	gtk_widget_show (priv->main_table);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), priv->main_table, TRUE, TRUE, 2);
 
-	priv->subscribe_button = glade_xml_get_widget (priv->xml, "subscribe-button");
-	g_signal_connect (G_OBJECT (priv->subscribe_button), "clicked",
-			    G_CALLBACK (subscribe_cb), dialog);
-	priv->unsubscribe_button = glade_xml_get_widget (priv->xml, "unsubscribe-button");
-	g_signal_connect (G_OBJECT (priv->unsubscribe_button), "clicked",
-			    G_CALLBACK (unsubscribe_cb), dialog);
+	priv->search_field = glade_xml_get_widget (priv->xml, "subscription-search-field");
+	gtk_combo_box_set_active ((GtkComboBox *) priv->search_field, 0);
+
+	g_signal_connect ((GtkComboBox *) priv->search_field,
+			    "changed", G_CALLBACK (search_activated_cb), dialog);
 
 	priv->search_by = glade_xml_get_widget (priv->xml, "subscription-search-by");
 	gtk_combo_box_set_active ((GtkComboBox *) priv->search_by, 0);
@@ -405,10 +263,6 @@ sunone_subscription_dialog_init (SunOneSubscriptionDialog *object)
 			    G_CALLBACK (clear_search_cb), dialog);
 
 	priv->subscriptions_tree = (GtkTreeView *)glade_xml_get_widget (priv->xml, "sub-treeview");
-	selection = gtk_tree_view_get_selection (priv->subscriptions_tree);
-	g_signal_connect (G_OBJECT (selection), "changed",
-			  G_CALLBACK (subscription_selection_changed_cb), dialog);
-
 
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), GTK_RESPONSE_OK, FALSE);
 }
@@ -444,12 +298,6 @@ sunone_subscription_dialog_dispose (GObject *object)
 			priv->xml = NULL;
 		}
 
-		if (priv->calids) {
-			g_list_foreach (priv->calids, (GFunc) sunone_connection_free_calprops, NULL);
-			g_list_free (priv->calids);
-			priv->calids = NULL;
-		}
-
 		if (priv->prefs) {
 			sunone_connection_free_preferences (priv->prefs);
 			priv->prefs = NULL;
@@ -461,6 +309,12 @@ sunone_subscription_dialog_dispose (GObject *object)
 			priv->to_subscribe = NULL;
 		}
 
+		if (priv->matched_calids) {
+			g_list_foreach (priv->matched_calids, (GFunc) sunone_connection_free_calprops, NULL);
+			g_list_free (priv->matched_calids);
+			priv->matched_calids = NULL;
+		}
+
 		g_free (priv);
 		dialog->priv = NULL;
 	}
@@ -544,6 +398,7 @@ update_folder_tree (SunOneSubscriptionDialog *dialog, GList *old_subscriptions)
 
 	/* now, the left ones are new subscriptions */
 	g_hash_table_foreach (priv->to_subscribe, (GHFunc) add_folder_hash, dialog);
+	sunone_folder_tree_sync ();
 }
 
 static void
@@ -598,22 +453,6 @@ load_subscriptions (SunOneSubscriptionDialog *dialog)
 	SunOneSubscriptionDialogPrivate *priv = dialog->priv;
 	GtkListStore *subscriptions_model;
 	GtkCellRenderer *renderer;
-	GList *l;
-	
-	/* get list of calids from server */
-	priv->calids = sunone_connection_list_calids (priv->cnc);
-	for (l = priv->calids; l != NULL; l = l->next)
-		priv->matched_calids = g_list_append (priv->matched_calids, l->data);
-
-	priv->prefs = sunone_connection_get_preferences (priv->cnc);
-	if (priv->prefs) {
-		for (l = priv->prefs->subscriptions; l != NULL; l = l->next) {
-			SunOneCalendarSubscription *sub = l->data;
-
-			g_hash_table_insert (priv->to_subscribe, g_strdup (sub->calid),
-					     g_strdup (sub->display_name ? sub->display_name : ""));
-		}
-	}
 
 	/* create the Table and its model */
 	subscriptions_model = gtk_list_store_new (SUB_TREE_N_COLUMNS, G_TYPE_BOOLEAN,
@@ -622,6 +461,8 @@ load_subscriptions (SunOneSubscriptionDialog *dialog)
 	gtk_tree_view_set_model (priv->subscriptions_tree, (GtkTreeModel *)subscriptions_model);
 	
 	renderer = gtk_cell_renderer_toggle_new();
+	g_signal_connect (renderer, "toggled",
+					    G_CALLBACK (subscribe_toggled), dialog);
 	gtk_tree_view_insert_column_with_attributes (priv->subscriptions_tree, -1,
 						     _("Subscription"), renderer,
 						     "active",
@@ -650,7 +491,7 @@ load_subscriptions (SunOneSubscriptionDialog *dialog)
 						     NULL);
 
 	if (priv->matched_calids)
-	       refresh_table (dialog);	
+		refresh_table (dialog);	
 }
 
 GtkWidget *
